@@ -16,6 +16,7 @@ package index
 import (
 	"context"
 	"fmt"
+	"hash/crc32"
 	"io/ioutil"
 	"math/rand"
 	"os"
@@ -37,18 +38,16 @@ type series struct {
 }
 
 type mockIndex struct {
-	series     map[uint64]series
-	labelIndex map[string][]string
-	postings   map[labels.Label][]uint64
-	symbols    map[string]struct{}
+	series   map[uint64]series
+	postings map[labels.Label][]uint64
+	symbols  map[string]struct{}
 }
 
 func newMockIndex() mockIndex {
 	ix := mockIndex{
-		series:     make(map[uint64]series),
-		labelIndex: make(map[string][]string),
-		postings:   make(map[labels.Label][]uint64),
-		symbols:    make(map[string]struct{}),
+		series:   make(map[uint64]series),
+		postings: make(map[labels.Label][]uint64),
+		symbols:  make(map[string]struct{}),
 	}
 	ix.postings[allPostingsKey] = []uint64{}
 	return ix
@@ -87,13 +86,14 @@ func (m mockIndex) Close() error {
 	return nil
 }
 
-func (m mockIndex) LabelValues(names ...string) (StringTuples, error) {
-	// TODO support composite indexes
-	if len(names) != 1 {
-		return nil, errors.New("composite indexes not supported yet")
+func (m mockIndex) LabelValues(name string) ([]string, error) {
+	values := []string{}
+	for l := range m.postings {
+		if l.Name == name {
+			values = append(values, l.Value)
+		}
 	}
-
-	return NewStringTuples(m.labelIndex[names[0]], 1)
+	return values, nil
 }
 
 func (m mockIndex) Postings(name string, values ...string) (Postings, error) {
@@ -221,7 +221,7 @@ func TestIndexRW_Postings(t *testing.T) {
 		vals := []string{}
 		nc := d.Be32int()
 		if nc != 1 {
-			return errors.Errorf("unexpected nuumber of label indices table names %d", nc)
+			return errors.Errorf("unexpected number of label indices table names %d", nc)
 		}
 		for i := d.Be32(); i > 0; i-- {
 			v, err := ir.lookupSymbol(d.Be32())
@@ -234,8 +234,8 @@ func TestIndexRW_Postings(t *testing.T) {
 		return d.Err()
 	}))
 	testutil.Equals(t, map[string][]string{
-		"a": []string{"1"},
-		"b": []string{"1", "2", "3", "4"},
+		"a": {"1"},
+		"b": {"1", "2", "3", "4"},
 	}, labelIndices)
 
 	testutil.Ok(t, ir.Close())
@@ -280,6 +280,7 @@ func TestPostingsMany(t *testing.T) {
 
 	ir, err := NewFileReader(fn)
 	testutil.Ok(t, err)
+	defer func() { testutil.Ok(t, ir.Close()) }()
 
 	cases := []struct {
 		in []string
@@ -445,22 +446,19 @@ func TestPersistence_index_e2e(t *testing.T) {
 		testutil.Ok(t, gotp.Err())
 	}
 
-	for k, v := range mi.labelIndex {
-		tplsExp, err := NewStringTuples(v, 1)
+	labelPairs := map[string][]string{}
+	for l := range mi.postings {
+		labelPairs[l.Name] = append(labelPairs[l.Name], l.Value)
+	}
+	for k, v := range labelPairs {
+		sort.Strings(v)
+
+		res, err := ir.LabelValues(k)
 		testutil.Ok(t, err)
 
-		tplsRes, err := ir.LabelValues(k)
-		testutil.Ok(t, err)
-
-		testutil.Equals(t, tplsExp.Len(), tplsRes.Len())
-		for i := 0; i < tplsExp.Len(); i++ {
-			strsExp, err := tplsExp.At(i)
-			testutil.Ok(t, err)
-
-			strsRes, err := tplsRes.At(i)
-			testutil.Ok(t, err)
-
-			testutil.Equals(t, strsExp, strsRes)
+		testutil.Equals(t, len(v), len(res))
+		for i := 0; i < len(v); i++ {
+			testutil.Equals(t, v[i], res[i])
 		}
 	}
 
@@ -480,7 +478,7 @@ func TestPersistence_index_e2e(t *testing.T) {
 	testutil.Ok(t, ir.Close())
 }
 
-func TestDecbufUvariantWithInvalidBuffer(t *testing.T) {
+func TestDecbufUvarintWithInvalidBuffer(t *testing.T) {
 	b := realByteSlice([]byte{0x81, 0x81, 0x81, 0x81, 0x81, 0x81})
 
 	db := encoding.NewDecbufUvarintAt(b, 0, castagnoliTable)
@@ -508,3 +506,50 @@ func TestNewFileReaderErrorNoOpenFiles(t *testing.T) {
 	// dir.Close will fail on Win if idxName fd is not closed on error path.
 	dir.Close()
 }
+
+func TestSymbols(t *testing.T) {
+	buf := encoding.Encbuf{}
+
+	// Add prefix to the buffer to simulate symbols as part of larger buffer.
+	buf.PutUvarintStr("something")
+
+	symbolsStart := buf.Len()
+	buf.PutBE32int(204) // Length of symbols table.
+	buf.PutBE32int(100) // Number of symbols.
+	for i := 0; i < 100; i++ {
+		// i represents index in unicode characters table.
+		buf.PutUvarintStr(string(i)) // Symbol.
+	}
+	checksum := crc32.Checksum(buf.Get()[symbolsStart+4:], castagnoliTable)
+	buf.PutBE32(checksum) // Check sum at the end.
+
+	s, err := NewSymbols(realByteSlice(buf.Get()), FormatV2, symbolsStart)
+	testutil.Ok(t, err)
+
+	// We store only 4 offsets to symbols.
+	testutil.Equals(t, 32, s.Size())
+
+	for i := 99; i >= 0; i-- {
+		s, err := s.Lookup(uint32(i))
+		testutil.Ok(t, err)
+		testutil.Equals(t, string(i), s)
+	}
+	_, err = s.Lookup(100)
+	testutil.NotOk(t, err)
+
+	for i := 99; i >= 0; i-- {
+		r, err := s.ReverseLookup(string(i))
+		testutil.Ok(t, err)
+		testutil.Equals(t, uint32(i), r)
+	}
+	_, err = s.ReverseLookup(string(100))
+	testutil.NotOk(t, err)
+
+	iter := s.Iter()
+	i := 0
+	for iter.Next() {
+		testutil.Equals(t, string(i), iter.At())
+		i++
+	}
+	testutil.Ok(t, iter.Err())
+}
